/*
	File:		TListExample.h

	Contains:	Declarations for the TPoint and TPointMatchObject classes used
				in TSimpleListExample.cp and TLinkedListExample.cp.

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef __TLISTEXAMPLE__
#define __TLISTEXAMPLE__

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif

///
///	TPoint
///
/// A simple class that contains a point.
///

class TPoint : public TSimpleDynamic {
	public:
								TPoint();
								TPoint(Point);
								TPoint(short h, short v);
		virtual					~TPoint();

		virtual	Point			GetPoint() const;
	private:	
		Point					fPoint;
};

///
///	TPoint IMLEMENTATION
///

	inline TPoint::TPoint()
	{
		fPoint.h = fPoint.v = 0;
	}

	inline TPoint::TPoint( Point thePoint)
	{
		fPoint = thePoint;
	}

	inline TPoint::TPoint(short h, short v)
	{
		SetPt( &fPoint, h, v );
	}

	TPoint::~TPoint()
	{
	}

	Point TPoint::GetPoint() const
	{
		return fPoint;
	}

///
///	TPointMatchObject
///
/// A match object that will compare the point stored in a TPoint object to the point
/// that the TPointMatchObject is initialized with.
///

class TPointMatchObject : public TMatchObject {
	public:
								TPointMatchObject();
								TPointMatchObject(Point);
								TPointMatchObject(TPoint*);
		virtual					~TPointMatchObject();

		virtual short			Compare(const void*) const;

	private:	
		Point					fPoint;
};

///
///	TPointMatchObject IMLEMENTATION
///

	inline TPointMatchObject::TPointMatchObject()
	{
		fPoint.h = fPoint.v = 0;
	}

	inline TPointMatchObject::TPointMatchObject( Point thePoint)
	{
		fPoint = thePoint;
	}

	inline TPointMatchObject::TPointMatchObject( TPoint* thePoint)
	{
		fPoint = thePoint->GetPoint();
	}

	TPointMatchObject::~TPointMatchObject()
	{
	}

	short TPointMatchObject::Compare(const void* object) const
	{
		//
		// Unfortunately we can't do couts here since Compare is always called
		// with interrupts disabled, so do Trace's instead.
		//

		Point point = ((const TPoint*)object)->GetPoint();

		Trace("TPointMatchObject::Compare called. point = (%d,%d)",
			int(point.h),int(point.v));
		if ( EqualPt(fPoint, point) )
		{
			Trace(" A match!\n");
			return 0;	// 0 means they are equal
		}
		else
		{
			Trace("\n");
			return -1;	// since there is no real ordering of Points we can return
						// 1 or -1
		}
	}

#endif;